@@ -1,14 +1,14 @@
 #!/bin/sh
 #
-# This is a driver installer for eGalax Touch controller.
+# This is a driver installer for eGalaxTouch controller.
 #
 # Company: eGalax_eMPIA Technology Inc. (EETI)
-# Version: 1.05.3128
-# Released: 2010/07/28
+# Version: 1.05.3712
+# Released: 2011/01/12
 # Support: touch_fae@eeti.com
 #
 
-version="1.05.3128"
+version="1.05.3712"
 driver="eGalaxTouch.tar.gz"
 utility="eGalaxTouch"
 caltool="TKCal"
@@ -32,10 +32,21 @@ libpath="/usr/lib/libXfixes.so.3"
 libpath2="/usr/X11R6/lib/libXfixes.so.3"
 rulefile="69-eeti_hiddev.rules"
 rulepath="/etc/udev/rules.d"
+# Added: 2010/09/06
+evdevfile="05-evdev.conf"
+tpadfile="10-synaptics.conf"
+xorgfile="10-egalax.conf" # for Ubuntu 10.04 & Meego 1.0
+xorgfile2="50-egalax.conf" # for Fedora 13
+xorgfile3="10-egalax-void.conf" # for Ubuntu 10.04 & Meego 1.0
+xorgfile4="50-egalax-void.conf" # for Fedora 13
+xorgpath="/usr/lib/X11/xorg.conf.d" # for Ubuntu 10.04 and Meego 1.0 netbook
+xorgpath2="/usr/share/X11/xorg.conf.d" # for Fedora 13/14 & Ubuntu 10.10
+xorgpath3="/etc/X11/xorg.conf.d"
+X11path="/usr/lib/X11"
 
 ShowTitle() {
     echo ""
-    echo "(*) Linux driver installer for eGalax Touch controller "
+    echo "(*) Linux driver installer for eGalaxTouch controller "
     echo ""
 }
 
@@ -58,16 +69,16 @@ CheckOldDaemon() {
     ${checkprocess} > ${processfile} 2> ${trash}
     grep -q "tpaneld" ${processfile}
     if [ $? -eq 0 ]; then
-        echo "(W) Found the old eGalax Touch daemon: tpaneld."
-        echo "(E) It has to be removed before run this installer."
+        echo "(W) Found the old eGalaxTouch daemon: tpaneld"
+        echo "(E) It has to be removed before running this installer."
         echo ""
         rm -f ${processfile}
         exit 1
     fi
     grep -q "usbpnpd" ${processfile}
     if [ $? -eq 0 ]; then
-        echo "(W) Found the old eGalax Touch daemon: usbpnpd."
-        echo "(E) It has to be removed before run this installer."
+        echo "(W) Found the old eGalaxTouch daemon: usbpnpd"
+        echo "(E) It has to be removed before running this installer."
         echo ""
         rm -f ${processfile}
         exit 1
@@ -80,10 +91,10 @@ CheckOldDriver() {
     olddriverpath=`find ${shortcutpath} -name ${olddriver}`
     if [ -z "${olddriverpath}" ]; then
         CheckOldDaemon
-        echo "(I) Begin to setup the eGalax Touch driver."
+        echo "(I) Begin to setup the eGalaxTouch driver."
     else
-        echo "(W) Found the old eGalax Touch driver."
-        echo "(E) It has to be removed before run this installer."
+        echo "(W) Found the old eGalaxTouch driver."
+        echo "(E) It has to be removed before running this installer."
         ${olddriver}
     fi
 }
@@ -96,21 +107,32 @@ ExtractDriver() {
     driverpath=${temppath%/*}
     if [ -n "${driverpath}" ]; then
         rm -rf ${driverpath}
-        echo "(I) Found and removed previous eGalax Touch driver."
+        echo "(I) Found and removed previous eGalaxTouch driver."
     fi
 
     cp -f ${driver} ${installpath}
     ( cd ${installpath}; tar -zxf ${driver}; rm -f ${driver} )
     temppath=`find ${installpath} -name ${utility}`
     driverpath=${temppath%/*}
-    echo "(I) Extract eGalax Touch driver archive to ${driverpath}."
+    echo "(I) Extract eGalaxTouch driver archive to ${driverpath}."
+
+    checkx="X -version" # Added: 2010/09/06
+    xfile="X.info"
+    ${checkx} > ${xfile} 2>&1
+    grep -q "X Window System Version" ${xfile}
+    if [ $? -eq 0 ]; then
+        fullpath=`find ${installpath} -name Utility_FC4`
+        toolpath=${fullpath%/*}
+        cp -f ${toolpath}/Utility_FC4 ${driverpath}/${utility}
+    fi
+    rm -f ${xfile}
 
     if [ -e ${libpath} ]; then
         rm -rf ${driverpath}/Tool
     elif [ -e ${libpath2} ]; then
         rm -rf ${driverpath}/Tool
     else
-        echo "(I) Searching the shared library: libXfixes.so.3."
+        echo "(I) Searching the shared library: libXfixes.so.3"
         sharedpath=`find /usr -name libXfixes.so.3`
         if [ -z "${sharedpath}" ]; then
             echo "(I) No shared library: libXfixes.so.3 found."
@@ -119,7 +141,7 @@ ExtractDriver() {
             cp -f ${toolpath}/Utility_FC2 ${driverpath}/${utility}
             cp -f ${toolpath}/Cal_FC2 ${driverpath}/TKCal/${caltool}
             rm -rf ${toolpath}
-            echo "(I) Replaced by another edition of utility."
+            echo "(I) Replaced by another build version of utility."
         else
             rm -rf ${driverpath}/Tool
         fi
@@ -150,7 +172,7 @@ RemoveDriver() {
         exit 1
     elif [ -n "${driverpath}" ]; then
         rm -rf ${driverpath}
-        echo "(I) Removed eGalax Touch driver archive from ${driverpath}."
+        echo "(I) Removed eGalaxTouch driver archive from ${driverpath}."
     fi
 
     if [ -d ${shortcutpath} ]; then
@@ -205,14 +227,20 @@ CheckXorgVersion() {
                             echo "(I) Check X window version: 1.8.x"
                             xdirectory="x18"
                         else
-                            grep -q "X Window System Version 6.8.99.900" ${xfile} # Mandriva 2006
+                            grep -q "X.Org X Server 1.9." ${xfile} # Fedora 14 & Ubuntu 10.10
                             if [ $? -eq 0 ]; then
-                                echo "(I) Check X window version: 6.8.99.900"
-                                xmodule="egalax_drv.o"
-                                xdirectory="x67"
+                                echo "(I) Check X window version: 1.9.x"
+                                xdirectory="x19"
                             else
-                                echo "(I) Check X window version: 6.9.0 ~ 7.2.0"
-                                xdirectory="x69"
+                                grep -q "X Window System Version 6.8.99.900" ${xfile} # Mandriva 2006
+                                if [ $? -eq 0 ]; then
+                                    echo "(I) Check X window version: 6.8.99.900"
+                                    xmodule="egalax_drv.o"
+                                    xdirectory="x67"
+                                else
+                                    echo "(I) Check X window version: 6.9.0 ~ 7.2.0"
+                                    xdirectory="x69"
+                                fi
                             fi
                         fi
                     fi
@@ -293,6 +321,14 @@ RemoveXorgModule() {
     echo "(I) Removed X module."
 }
 
+CopyUdevRule() {
+    if [ -e ${rulepath}/${rulefile} ]; then
+        rm -f ${rulepath}/${rulefile}
+    fi
+    echo "(I) Copy udev rule: ${rulefile} to ${rulepath}."
+    cp -f ${driverpath}/Rule/${rulefile} ${rulepath}
+}
+
 CheckMoblin() {
     checkx="X -version"
     xfile="X.info"
@@ -301,18 +337,22 @@ CheckMoblin() {
     if [ $? -eq 0 ]; then
         echo "(I) Check Linux distribution: Moblin"
         CopyUdevRule
+    else
+        grep -q "Build ID: xorg-x11-server 1.8.0-5.14" ${xfile} # for Meego 1.0 netbook
+        if [ $? -eq 0 ]; then
+            echo "(I) Check Linux distribution: Meego 1.0"
+            CopyUdevRule
+        else
+            grep -q "Build ID: xorg-x11-server 1.9.0-4.6" ${xfile} # for Meego 1.1 netbook
+            if [ $? -eq 0 ]; then
+                echo "(I) Check Linux distribution: Meego 1.1"
+                CopyUdevRule
+            fi
+        fi
     fi
     rm -f ${xfile}
 }
 
-CopyUdevRule() {
-    if [ -e ${rulepath}/${rulefile} ]; then
-        rm -f ${rulepath}/${rulefile}
-    fi
-    echo "(I) Copy udev rule: ${driverpath}/Rule/${rulefile} to ${rulepath}."
-    cp -f ${driverpath}/Rule/${rulefile} ${rulepath}
-}
-
 RemoveUdevRule() {
     if [ -e ${rulepath}/${rulefile} ]; then
         rm -f ${rulepath}/${rulefile}
@@ -326,7 +366,7 @@ Add2Blacklist() {
         if [ $? -eq 1 ]; then
             filelines=`cat ${blacklistpath} | wc -l`
             echo "(I) Add kernel module $1 into ${blacklistpath}."
-            sed -i ''${filelines}'a\# added by eGalax Touch installer\nblacklist '$1'' ${blacklistpath}
+            sed -i ''${filelines}'a\# added by eGalaxTouch installer\nblacklist '$1'' ${blacklistpath}
         else
             echo "(I) The kernel module $1 has been added in ${blacklistpath}."
         fi
@@ -335,7 +375,7 @@ Add2Blacklist() {
         if [ $? -eq 1 ]; then
             filelines=`cat ${blacklistpath2} | wc -l`
             echo "(I) Add kernel module $1 into ${blacklistpath2}."
-            sed -i ''${filelines}'a\# added by eGalax Touch installer\nblacklist '$1'' ${blacklistpath2}
+            sed -i ''${filelines}'a\# added by eGalaxTouch installer\nblacklist '$1'' ${blacklistpath2}
         else
             echo "(I) The kernel module $1 has been added in ${blacklistpath2}."
         fi
@@ -344,7 +384,7 @@ Add2Blacklist() {
         if [ $? -eq 1 ]; then
             filelines=`cat ${blacklistpath3} | wc -l`
             echo "(I) Add kernel module $1 into ${blacklistpath3}."
-            sed -i ''${filelines}'a\# added by eGalax Touch installer\nblacklist '$1'' ${blacklistpath3}
+            sed -i ''${filelines}'a\# added by eGalaxTouch installer\nblacklist '$1'' ${blacklistpath3}
         else
             echo "(I) The kernel module $1 has been added in ${blacklistpath3}."
         fi
@@ -353,7 +393,7 @@ Add2Blacklist() {
         if [ $? -eq 1 ]; then
             filelines=`cat ${blacklistpath4} | wc -l`
             echo "(I) Add kernel module $1 into ${blacklistpath4}."
-            sed -i ''${filelines}'a\# added by eGalax Touch installer\nblacklist '$1'' ${blacklistpath4}
+            sed -i ''${filelines}'a\# added by eGalaxTouch installer\nblacklist '$1'' ${blacklistpath4}
         else
             echo "(I) The kernel module $1 has been added in ${blacklistpath4}."
         fi
@@ -362,7 +402,7 @@ Add2Blacklist() {
         if [ $? -eq 1 ]; then
             filelines=`cat ${blacklistpath5} | wc -l`
             echo "(I) Add kernel module $1 into ${blacklistpath5}."
-            sed -i ''${filelines}'a\# added by eGalax Touch installer\n'$1'' ${blacklistpath5}
+            sed -i ''${filelines}'a\# added by eGalaxTouch installer\n'$1'' ${blacklistpath5}
         else
             echo "(I) The kernel module $1 has been added in ${blacklistpath5}."
         fi
@@ -373,7 +413,7 @@ Add2Blacklist() {
             if [ $? -eq 1 ]; then
                 filelines=`cat ${rclocalpath} | wc -l`
                 echo "(I) Add kernel module $1 into ${rclocalpath}."
-                sed -i ''${filelines}'a\# added by eGalax Touch installer\nrmmod '$1'' ${rclocalpath}
+                sed -i ''${filelines}'a\# added by eGalaxTouch installer\nrmmod '$1'' ${rclocalpath}
             else
                 echo "(I) The kernel module $1 has been added in ${rclocalpath}."
             fi
@@ -405,12 +445,12 @@ CheckModuleAndBlacklist() {
     ${checkmod} > ${modfile} 2> ${trash}
     grep -q "usbtouchscreen" ${modfile}
     if [ $? -eq 0 ]; then
-        echo "(I) Found inbuilt kernel module: usbtouchscreen."
+        echo "(I) Found inbuilt kernel module: usbtouchscreen"
         ShowBlacklistMenu "usbtouchscreen"
     else
         grep -q "touchkitusb" ${modfile}
         if [ $? -eq 0 ]; then
-            echo "(I) Found inbuilt kernel module: touchkitusb."
+            echo "(I) Found inbuilt kernel module: touchkitusb"
             ShowBlacklistMenu "touchkitusb"
         fi
     fi
@@ -425,7 +465,7 @@ CheckInbuiltModule() {
     if [ $? -eq 1 ]; then
         grep -q "touchkitusb" ${modfile}
         if [ $? -eq 1 ]; then
-            echo "(W) No inbuilt kernel module for touch controller found."
+            echo "(W) No inbuilt kernel module of touch controller found."
             echo "(I) It is needed to build \"tkusb\" kernel module for touch controller."
             echo "(I) For details, see the document \"How to build module.pdf\"."
         else
@@ -502,17 +542,17 @@ RestoreBlacklist() {
     if [ -w ${blacklistpath} ]; then
         grep -q "blacklist usbtouchscreen" ${blacklistpath}
         if [ $? -eq 0 ]; then
-            grep -q "# added by eGalax Touch installer" ${blacklistpath}
+            grep -q "# added by eGalaxTouch installer" ${blacklistpath}
             if [ $? -eq 0 ]; then
-                sed -i '/# added by eGalax Touch installer/,/blacklist usbtouchscreen/d' ${blacklistpath}
+                sed -i '/# added by eGalaxTouch installer/,/blacklist usbtouchscreen/d' ${blacklistpath}
                 echo "(I) Removed blacklist usbtouchscreen from ${blacklistpath}."
             fi
         else
             grep -q "blacklist touchkitusb" ${blacklistpath}
             if [ $? -eq 0 ]; then
-                grep -q "# added by eGalax Touch installer" ${blacklistpath}
+                grep -q "# added by eGalaxTouch installer" ${blacklistpath}
                 if [ $? -eq 0 ]; then
-                    sed -i '/# added by eGalax Touch installer/,/blacklist touchkitusb/d' ${blacklistpath}
+                    sed -i '/# added by eGalaxTouch installer/,/blacklist touchkitusb/d' ${blacklistpath}
                     echo "(I) Removed blacklist touchkitusb from ${blacklistpath}."
                 fi
             fi
@@ -520,17 +560,17 @@ RestoreBlacklist() {
     elif [ -w ${blacklistpath2} ]; then
         grep -q "blacklist usbtouchscreen" ${blacklistpath2}
         if [ $? -eq 0 ]; then
-            grep -q "# added by eGalax Touch installer" ${blacklistpath2}
+            grep -q "# added by eGalaxTouch installer" ${blacklistpath2}
             if [ $? -eq 0 ]; then
-                sed -i '/# added by eGalax Touch installer/,/blacklist usbtouchscreen/d' ${blacklistpath2}
+                sed -i '/# added by eGalaxTouch installer/,/blacklist usbtouchscreen/d' ${blacklistpath2}
                 echo "(I) Removed blacklist usbtouchscreen from ${blacklistpath2}."
             fi
         else
             grep -q "blacklist touchkitusb" ${blacklistpath2}
             if [ $? -eq 0 ]; then
-                grep -q "# added by eGalax Touch installer" ${blacklistpath2}
+                grep -q "# added by eGalaxTouch installer" ${blacklistpath2}
                 if [ $? -eq 0 ]; then
-                    sed -i '/# added by eGalax Touch installer/,/blacklist touchkitusb/d' ${blacklistpath2}
+                    sed -i '/# added by eGalaxTouch installer/,/blacklist touchkitusb/d' ${blacklistpath2}
                     echo "(I) Removed blacklist touchkitusb from ${blacklistpath2}."
                 fi
             fi
@@ -538,17 +578,17 @@ RestoreBlacklist() {
     elif [ -w ${blacklistpath3} ]; then
         grep -q "blacklist usbtouchscreen" ${blacklistpath3}
         if [ $? -eq 0 ]; then
-            grep -q "# added by eGalax Touch installer" ${blacklistpath3}
+            grep -q "# added by eGalaxTouch installer" ${blacklistpath3}
             if [ $? -eq 0 ]; then
-                sed -i '/# added by eGalax Touch installer/,/blacklist usbtouchscreen/d' ${blacklistpath3}
+                sed -i '/# added by eGalaxTouch installer/,/blacklist usbtouchscreen/d' ${blacklistpath3}
                 echo "(I) Removed blacklist usbtouchscreen from ${blacklistpath3}."
             fi
         else
             grep -q "blacklist touchkitusb" ${blacklistpath3}
             if [ $? -eq 0 ]; then
-                grep -q "# added by eGalax Touch installer" ${blacklistpath3}
+                grep -q "# added by eGalaxTouch installer" ${blacklistpath3}
                 if [ $? -eq 0 ]; then
-                    sed -i '/# added by eGalax Touch installer/,/blacklist touchkitusb/d' ${blacklistpath3}
+                    sed -i '/# added by eGalaxTouch installer/,/blacklist touchkitusb/d' ${blacklistpath3}
                     echo "(I) Removed blacklist touchkitusb from ${blacklistpath3}."
                 fi
             fi
@@ -556,17 +596,17 @@ RestoreBlacklist() {
     elif [ -w ${blacklistpath4} ]; then
         grep -q "blacklist usbtouchscreen" ${blacklistpath4}
         if [ $? -eq 0 ]; then
-            grep -q "# added by eGalax Touch installer" ${blacklistpath4}
+            grep -q "# added by eGalaxTouch installer" ${blacklistpath4}
             if [ $? -eq 0 ]; then
-                sed -i '/# added by eGalax Touch installer/,/blacklist usbtouchscreen/d' ${blacklistpath4}
+                sed -i '/# added by eGalaxTouch installer/,/blacklist usbtouchscreen/d' ${blacklistpath4}
                 echo "(I) Removed blacklist usbtouchscreen from ${blacklistpath4}."
             fi
         else
             grep -q "blacklist touchkitusb" ${blacklistpath4}
             if [ $? -eq 0 ]; then
-                grep -q "# added by eGalax Touch installer" ${blacklistpath4}
+                grep -q "# added by eGalaxTouch installer" ${blacklistpath4}
                 if [ $? -eq 0 ]; then
-                    sed -i '/# added by eGalax Touch installer/,/blacklist touchkitusb/d' ${blacklistpath4}
+                    sed -i '/# added by eGalaxTouch installer/,/blacklist touchkitusb/d' ${blacklistpath4}
                     echo "(I) Removed blacklist touchkitusb from ${blacklistpath4}."
                 fi
             fi
@@ -574,17 +614,17 @@ RestoreBlacklist() {
     elif [ -w ${blacklistpath5} ]; then
         grep -q "usbtouchscreen" ${blacklistpath5}
         if [ $? -eq 0 ]; then
-            grep -q "# added by eGalax Touch installer" ${blacklistpath5}
+            grep -q "# added by eGalaxTouch installer" ${blacklistpath5}
             if [ $? -eq 0 ]; then
-                sed -i '/# added by eGalax Touch installer/,/usbtouchscreen/d' ${blacklistpath5}
+                sed -i '/# added by eGalaxTouch installer/,/usbtouchscreen/d' ${blacklistpath5}
                 echo "(I) Removed usbtouchscreen from ${blacklistpath5}."
             fi
         else
             grep -q "touchkitusb" ${blacklistpath5}
             if [ $? -eq 0 ]; then
-                grep -q "# added by eGalax Touch installer" ${blacklistpath5}
+                grep -q "# added by eGalaxTouch installer" ${blacklistpath5}
                 if [ $? -eq 0 ]; then
-                    sed -i '/# added by eGalax Touch installer/,/touchkitusb/d' ${blacklistpath5}
+                    sed -i '/# added by eGalaxTouch installer/,/touchkitusb/d' ${blacklistpath5}
                     echo "(I) Removed touchkitusb from ${blacklistpath5}."
                 fi
             fi
@@ -592,17 +632,17 @@ RestoreBlacklist() {
     elif [ -w ${rclocalpath} ]; then
         grep -q "rmmod usbtouchscreen" ${rclocalpath}
         if [ $? -eq 0 ]; then
-            grep -q "# added by eGalax Touch installer" ${rclocalpath}
+            grep -q "# added by eGalaxTouch installer" ${rclocalpath}
             if [ $? -eq 0 ]; then
-                sed -i '/# added by eGalax Touch installer/,/rmmod usbtouchscreen/d' ${rclocalpath}
+                sed -i '/# added by eGalaxTouch installer/,/rmmod usbtouchscreen/d' ${rclocalpath}
                 echo "(I) Removed rmmod usbtouchscreen from ${rclocalpath}."
             fi
         else
             grep -q "rmmod touchkitusb" ${rclocalpath}
             if [ $? -eq 0 ]; then
-                grep -q "# added by eGalax Touch installer" ${rclocalpath}
+                grep -q "# added by eGalaxTouch installer" ${rclocalpath}
                 if [ $? -eq 0 ]; then
-                    sed -i '/# added by eGalax Touch installer/,/rmmod touchkitusb/d' ${rclocalpath}
+                    sed -i '/# added by eGalaxTouch installer/,/rmmod touchkitusb/d' ${rclocalpath}
                     echo "(I) Removed rmmod touchkitusb from ${rclocalpath}."
                 fi
             fi
@@ -691,15 +731,125 @@ RemoveConfiguration() {
     echo "(I) Removed touch configuration from $1."
 }
 
+CheckAndLoadVoid() {
+    checkx="X -version"
+    xfile="X.info"
+    ${checkx} > ${xfile} 2>&1
+    grep -q "xorg-server 2:1.7.6-2ubuntu7.3" ${xfile} # for Ubuntu 10.04
+    if [ $? -eq 0 ]; then
+        echo "(I) Copy udev rule: ${xorgfile3} to ${xorgpath}."
+        cp -f ${driverpath}/Rule/${xorgfile3} ${xorgpath}
+    else
+        grep -q "X.Org X Server 1.8." ${xfile}
+        if [ $? -eq 0 ]; then
+            grep -q "Build ID: xorg-x11-server 1.8.0-5.14" ${xfile} # for Meego 1.0 netbook
+            if [ $? -eq 0 ]; then
+                echo "(I) Copy udev rule: ${xorgfile3} to ${xorgpath}."
+                ( cd ${X11path}; mkdir xorg.conf.d )
+                cp -f ${driverpath}/Rule/${evdevfile} ${xorgpath}
+                cp -f ${driverpath}/Rule/${tpadfile} ${xorgpath}
+                cp -f ${driverpath}/Rule/${xorgfile3} ${xorgpath}
+            else
+                grep -q "Build Operating System: openSUSE SUSE LINUX" ${xfile}
+                if [ $? -eq 0 ]; then
+                    echo "(I) Copy udev rule: ${xorgfile4} to ${xorgpath3}."
+                    cp -f ${driverpath}/Rule/${xorgfile4} ${xorgpath3}
+                else
+                    #grep -q "Build ID: xorg-x11-server 1.8.0-12.fc13" ${xfile} # for Fedora 13
+                    #if [ $? -eq 0 ]; then
+                        echo "(I) Copy udev rule: ${xorgfile4} to ${xorgpath2}."
+                        cp -f ${driverpath}/Rule/${xorgfile4} ${xorgpath2}
+                    #fi
+                fi
+            fi
+        else
+            grep -q "X.Org X Server 1.9." ${xfile}
+            if [ $? -eq 0 ]; then
+                grep -q "Build ID: xorg-x11-server 1.9.0-4.6" ${xfile} # for Meego 1.1 netbook
+                if [ $? -eq 0 ]; then
+                    echo "(I) Copy udev rule: ${xorgfile4} to ${xorgpath3}."
+                    cp -f ${driverpath}/Rule/${xorgfile4} ${xorgpath3}
+                else
+                    echo "(I) Copy udev rule: ${xorgfile4} to ${xorgpath2}." # for Ubuntu 10.10 & Fedora 14
+                    cp -f ${driverpath}/Rule/${xorgfile4} ${xorgpath2}
+                fi
+            fi
+        fi
+    fi
+    rm -f ${xfile}
+}
+
+CheckAndLoadDrv() {
+    checkx="X -version"
+    xfile="X.info"
+    ${checkx} > ${xfile} 2>&1
+    grep -q "xorg-server 2:1.7.6-2ubuntu7.3" ${xfile} # for Ubuntu 10.04
+    if [ $? -eq 0 ]; then
+        echo "(I) Copy udev rule: ${xorgfile} to ${xorgpath}."
+        cp -f ${driverpath}/Rule/${xorgfile} ${xorgpath}
+    else
+        grep -q "X.Org X Server 1.8." ${xfile}
+        if [ $? -eq 0 ]; then
+            grep -q "Build ID: xorg-x11-server 1.8.0-5.14" ${xfile} # for Meego 1.0 netbook
+            if [ $? -eq 0 ]; then
+                echo "(I) Copy udev rule: ${xorgfile} to ${xorgpath}."
+                ( cd ${X11path}; mkdir xorg.conf.d )
+                cp -f ${driverpath}/Rule/${evdevfile} ${xorgpath}
+                cp -f ${driverpath}/Rule/${tpadfile} ${xorgpath}
+                cp -f ${driverpath}/Rule/${xorgfile} ${xorgpath}
+            else
+                grep -q "Build Operating System: openSUSE SUSE LINUX" ${xfile}
+                if [ $? -eq 0 ]; then
+                    echo "(I) Copy udev rule: ${xorgfile2} to ${xorgpath3}."
+                    cp -f ${driverpath}/Rule/${xorgfile2} ${xorgpath3}
+                else
+                    #grep -q "Build ID: xorg-x11-server 1.8.0-12.fc13" ${xfile} # for Fedora 13
+                    #if [ $? -eq 0 ]; then
+                        echo "(I) Copy udev rule: ${xorgfile2} to ${xorgpath2}."
+                        cp -f ${driverpath}/Rule/${xorgfile2} ${xorgpath2}
+                    #fi
+                fi
+            fi
+        else
+            grep -q "X.Org X Server 1.9." ${xfile}
+            if [ $? -eq 0 ]; then
+                grep -q "Build ID: xorg-x11-server 1.9.0-4.6" ${xfile} # for Meego 1.1 netbook
+                if [ $? -eq 0 ]; then
+                    echo "(I) Copy udev rule: ${xorgfile2} to ${xorgpath3}."
+                    cp -f ${driverpath}/Rule/${xorgfile2} ${xorgpath3}
+                else
+                    echo "(I) Copy udev rule: ${xorgfile2} to ${xorgpath2}." # for Ubuntu 10.10 & Fedora 14
+                    cp -f ${driverpath}/Rule/${xorgfile2} ${xorgpath2}
+                fi
+            else
+                RemoveDriver
+                RemoveXorgModule
+                RestoreBlacklist
+                rm -f ${xfile}
+                echo "(E) No X configuration file found."
+                echo ""
+                exit 1
+            fi
+        fi
+    fi
+    rm -f ${xfile}
+}
+
 EditConfigFile() {
     if [ -w ${xfilepath} ]; then
         echo "(I) Found X configuration file: ${xfilepath}."
         grep -q "### Touch Configuration Beginning ###" ${xfilepath}
         if [ $? -eq 1 ]; then
             AddConfiguration ${xfilepath} ${device}
+            if [ ${device} = "usbauto" ]; then # Added: 2011/01/12
+                CheckAndLoadVoid
+            fi
         else
             RemoveConfiguration ${xfilepath}
             AddConfiguration ${xfilepath} ${device}
+            if [ ${device} = "usbauto" ]; then # Added: 2011/01/12
+                CheckAndLoadVoid
+            fi
         fi
     elif [ -w ${xfilepath2} ]; then
         echo "(I) Found X configuration file: ${xfilepath2}."
@@ -719,6 +869,8 @@ EditConfigFile() {
             RemoveConfiguration ${xfilepath3}
             AddConfiguration ${xfilepath3} ${device}
         fi
+    elif [ ${device} = "usbauto" ]; then # Added: 2010/10/01
+        CheckAndLoadDrv
     else
         RemoveDriver
         RemoveXorgModule
@@ -729,17 +881,137 @@ EditConfigFile() {
     fi
 }
 
+RemoveVoidConfig() {
+    checkx="X -version" # Added: 2011/01/12
+    xfile="X.info"
+    ${checkx} > ${xfile} 2>&1
+    grep -q "xorg-server 2:1.7.6-2ubuntu7.3" ${xfile} # for Ubuntu 10.04
+    if [ $? -eq 0 ]; then
+        if [ -e ${xorgpath}/${xorgfile3} ]; then
+            rm -f ${xorgpath}/${xorgfile3}
+            echo "(I) Removed udev rule: ${xorgpath}/${xorgfile3}."
+        fi
+    else
+        grep -q "X.Org X Server 1.8." ${xfile}
+        if [ $? -eq 0 ]; then
+            grep -q "Build ID: xorg-x11-server 1.8.0-5.14" ${xfile} # for Meego 1.0 netbook
+            if [ $? -eq 0 ]; then
+                if [ -e ${xorgpath}/${xorgfile3} ]; then
+                    ( cd ${X11path}; rm -rf xorg.conf.d )
+                    echo "(I) Removed udev rule: ${xorgpath}/${xorgfile3}."
+                fi
+            else
+                grep -q "Build Operating System: openSUSE SUSE LINUX" ${xfile}
+                if [ $? -eq 0 ]; then
+                    if [ -e ${xorgpath3}/${xorgfile4} ]; then
+                        rm -f ${xorgpath3}/${xorgfile4}
+                        echo "(I) Removed udev rule: ${xorgpath3}/${xorgfile4}."
+                    fi
+                else
+                    #grep -q "Build ID: xorg-x11-server 1.8.0-12.fc13" ${xfile} # for Fedora 13
+                    #if [ $? -eq 0 ]; then
+                        if [ -e ${xorgpath2}/${xorgfile4} ]; then
+                            rm -f ${xorgpath2}/${xorgfile4}
+                            echo "(I) Removed udev rule: ${xorgpath2}/${xorgfile4}."
+                        fi
+                    #fi
+                fi
+            fi
+        else
+            grep -q "X.Org X Server 1.9." ${xfile}
+            if [ $? -eq 0 ]; then
+                grep -q "Build ID: xorg-x11-server 1.9.0-4.6" ${xfile} # for Meego 1.1 netbook
+                if [ $? -eq 0 ]; then
+                    if [ -e ${xorgpath3}/${xorgfile4} ]; then
+                        rm -f ${xorgpath3}/${xorgfile4}
+                        echo "(I) Removed udev rule: ${xorgpath3}/${xorgfile4}."
+                    fi
+                else
+                    if [ -e ${xorgpath2}/${xorgfile4} ]; then
+                        rm -f ${xorgpath2}/${xorgfile4}
+                        echo "(I) Removed udev rule: ${xorgpath2}/${xorgfile4}."
+                    fi
+                fi
+            fi
+        fi
+    fi
+    rm -f ${xfile}
+}
+
+RemoveDrvConfig() {
+    rm -f ${paramfile}
+
+    checkx="X -version" # Added: 2010/09/06
+    xfile="X.info"
+    ${checkx} > ${xfile} 2>&1
+    grep -q "xorg-server 2:1.7.6-2ubuntu7.3" ${xfile} # for Ubuntu 10.04
+    if [ $? -eq 0 ]; then
+        if [ -e ${xorgpath}/${xorgfile} ]; then
+            rm -f ${xorgpath}/${xorgfile}
+            echo "(I) Removed udev rule: ${xorgpath}/${xorgfile}."
+        fi
+    else
+        grep -q "X.Org X Server 1.8." ${xfile}
+        if [ $? -eq 0 ]; then
+            grep -q "Build ID: xorg-x11-server 1.8.0-5.14" ${xfile} # for Meego 1.0 netbook
+            if [ $? -eq 0 ]; then
+                if [ -e ${xorgpath}/${xorgfile} ]; then
+                    ( cd ${X11path}; rm -rf xorg.conf.d )
+                    echo "(I) Removed udev rule: ${xorgpath}/${xorgfile}."
+                fi
+            else
+                grep -q "Build Operating System: openSUSE SUSE LINUX" ${xfile}
+                if [ $? -eq 0 ]; then
+                    if [ -e ${xorgpath3}/${xorgfile2} ]; then
+                        rm -f ${xorgpath3}/${xorgfile2}
+                        echo "(I) Removed udev rule: ${xorgpath3}/${xorgfile2}."
+                    fi
+                else
+                    #grep -q "Build ID: xorg-x11-server 1.8.0-12.fc13" ${xfile} # for Fedora 13
+                    #if [ $? -eq 0 ]; then
+                        if [ -e ${xorgpath2}/${xorgfile2} ]; then
+                            rm -f ${xorgpath2}/${xorgfile2}
+                            echo "(I) Removed udev rule: ${xorgpath2}/${xorgfile2}."
+                        fi
+                    #fi
+                fi
+            fi
+        else
+            grep -q "X.Org X Server 1.9." ${xfile}
+            if [ $? -eq 0 ]; then
+                grep -q "Build ID: xorg-x11-server 1.9.0-4.6" ${xfile} # for Meego 1.1 netbook
+                if [ $? -eq 0 ]; then
+                    if [ -e ${xorgpath3}/${xorgfile2} ]; then
+                        rm -f ${xorgpath3}/${xorgfile2}
+                        echo "(I) Removed udev rule: ${xorgpath3}/${xorgfile2}."
+                    fi
+                else
+                    if [ -e ${xorgpath2}/${xorgfile2} ]; then
+                        rm -f ${xorgpath2}/${xorgfile2}
+                        echo "(I) Removed udev rule: ${xorgpath2}/${xorgfile2}."
+                    fi
+                fi
+            else
+                rm -f ${xfile}
+                echo "(E) No X configuration file found."
+                echo ""
+                exit 1
+            fi
+        fi
+    fi
+    rm -f ${xfile}
+}
+
 RestoreConfigFile() {
     if [ -w ${xfilepath} ]; then
         RemoveConfiguration ${xfilepath}
+        RemoveVoidConfig # Added: 2011/01/12
     elif [ -w ${xfilepath2} ]; then
         RemoveConfiguration ${xfilepath2}
     elif [ -w ${xfilepath3} ]; then
         RemoveConfiguration ${xfilepath3}
     else
-        echo "(E) No X configuration file found."
-        echo ""
-        exit 1
+        RemoveDrvConfig
     fi
 }
 
@@ -760,14 +1032,14 @@ if [ $# = 0 ]; then
 elif [ $# = 1 ]; then
     if [ $1 = "uninstall" ]; then
         CheckPermission
-        echo "(I) Begin to remove eGalax Touch driver."
+        echo "(I) Begin to remove eGalaxTouch driver."
         RemoveDriver
         RemoveXorgModule
         RemoveUdevRule
         RestoreBlacklist
         RestoreConfigFile
         echo ""
-        echo "(I) The eGalax Touch driver has been removed successfully."
+        echo "(I) The eGalaxTouch driver has been removed successfully."
         echo "(I) Please reboot the system for some changes to take effect."
     elif [ $1 = "version" ]; then
         echo "(I) Version: ${version}"
